#include "include.h"
#include "FAT16.h"
#include <def21369.h>
#include <cdef21369.h>
#include <sru.h>


void main()
{
	
    //Initializing the PLL and SDRAM controller

    
    
        InitPLL_SDRAM();
    
    //Initializing the SRU 
    init_SRU();
    
    //Initializing the MMC 
    mmc_init();
	
    // As initialization is done, baud rate could be increased.
	*pSPIBAUD = 80; // BAUDR = 150 => SPI Baud Rate = 331.776 / (8*40) = 1.0368 MHz
	
	// Set the block length - CMD16
	mmc_set_block_length(512);
	
	int command=0;

	//Initializing the internal memory with the FAT info.
	GetFATInfo();
	
	// Continously refresh the screen with the options available to the user till the user wishes
    // to exit from the program
    while(!halt)
    {
    	CommandInterface();
    }
}



//PLL and SDRAM initialization
void InitPLL_SDRAM()
{

	int i, pmctlsetting;

	#define SDMODIFY 1

    pmctlsetting= *pPMCTL;
    pmctlsetting &= ~(0xFF); //Clear

    // CLKIN= 24.576 MHz, Multiplier= 27, Divisor= 2, CCLK_SDCLK_RATIO 2.5
    // Core clock = (24.576 MHz * 27) /2 = 331.776 MHz
    pmctlsetting= SDCKR2_5|PLLM27|INDIV|DIVEN;
    *pPMCTL= pmctlsetting;
    pmctlsetting|= PLLBP;
    pmctlsetting^= DIVEN;
    *pPMCTL= pmctlsetting;

    //Wait for around 4096 cycles for the pll to lock.
    for (i=0; i<4096; i++)
          asm("nop;");

    *pPMCTL ^= PLLBP;       //Clear Bypass Mode
    *pPMCTL |= (CLKOUTEN);  //and start clkout
   
    *pSDRRC= (0x814)|(SDMODIFY<<17)|SDROPT;

    //===================================================================
    //
    // Configure SDRAM Control Register (SDCTL) for PART MT48LC4M32B2
    //
    //  SDCL3  : SDRAM CAS Latency= 3 cycles
    //  DSDCLK1: Disable SDRAM Clock 1
    //  SDPSS  : Start SDRAM Power up Sequence
    //  SDCAW8 : SDRAM Bank Column Address Width= 8 bits
    //  SDRAW12: SDRAM Row Address Width= 12 bits
    //  SDTRAS7: SDRAM tRAS Specification. Active Command delay = 7 cycles
    //  SDTRP3 : SDRAM tRP Specification. Precharge delay = 3 cycles.
    //  SDTWR2 : SDRAM tWR Specification. tWR = 2 cycles.
    //  SDTRCD3: SDRAM tRCD Specification. tRCD = 3 cycles.
    //
    //--------------------------------------------------------------------

    *pSDCTL= SDCL3|DSDCLK1|SDPSS|SDCAW8|SDRAW12|SDTRAS7|SDTRP3|SDTWR2|SDTRCD3;

    // Note that MS2 & MS3 pin multiplexed with flag2 & flag3.
    // MSEN bit must be enabled to access SDRAM, but LED7 cannot be driven with sdram
    *pSYSCTL |=MSEN;

    // Mapping Bank 2 to SDRAM
    // Make sure that jumper is set appropriately so that MS2 is connected to
    // chip select of 16-bit SDRAM device
    *pEPCTL |=B2SD;
    *pEPCTL &= ~(B0SD|B1SD|B3SD);

    //===================================================================
    //
    // Configure AMI Control Register (AMICTL0) Bank 0 for the ISSI IS61LV5128
    //
    //  WS2 : Wait States = 2 cycles
    //  HC1  : Bus Hold Cycle (at end of write access)= 1 cycle.
    //  AMIEN: Enable AMI
    //  BW8  : External Data Bus Width= 8 bits.
    //
    //--------------------------------------------------------------------

    //SRAM Settings
    *pAMICTL0 = WS2|HC1|AMIEN|BW8;

    //===================================================================
    //
    // Configure AMI Control Register (AMICTL) Bank 1 for the AMD AM29LV08
    //
    //  WS23 : Wait States= 23 cycles
    //  AMIEN: Enable AMI
    //  BW8  : External Data Bus Width= 8 bits.
    //
    //--------------------------------------------------------------------

    //Flash Settings
    *pAMICTL1 = WS23|AMIEN|BW8;

}


//SRU configuration
void init_SRU()
{

	//CS
	SRU(HIGH, DPI_PB06_I);
	SRU(HIGH, DPI_PBEN06_I);
	
	//CLK
	SRU(SPI_CLK_O, DPI_PB03_I);
	SRU(SPI_CLK_PBEN_O, DPI_PBEN03_I);

	//MOSI
	SRU(SPI_MOSI_O, DPI_PB11_I);
	SRU(HIGH, DPI_PBEN11_I);

	//MISO
	SRU(DPI_PB12_O, SPI_MISO_I);
	SRU(LOW, DPI_PBEN12_I);

	//trigerring flag
	SRU(LOW, DAI_PB10_I);
	SRU(HIGH, PBEN10_I);


	//Make other chip selects connected to the SPI FLASH high
	SRU(HIGH, DPI_PB04_I);
	SRU(HIGH, DPI_PBEN04_I);
	SRU(HIGH, DPI_PB05_I);
	SRU(HIGH, DPI_PBEN05_I);

}


//Command description
void help()
{
    printf("\n\nList of commands available:\n");
    printf("=================================================================================================================================================\n\n");
    printf("S.No.\t\tCommand Name\t\t\t\t\t\t\t\tFunction\n\n");
    printf("1.\t\t\t\"dir\"\t\t\t\t\t\t\t\t\t\tLists all the folders and files in the current Directory\n");
    printf("2.\t\t\t\"cd\"\t\t\t\t\t\t\t\t\t\tChanges the current directory to any of the sub-folders of the current directory\n");
    printf("3.\t\t\t\"cd..\"\t\t\t\t\t\t\t\t\t\tChanges the current directory to one level up\n");
    printf("4.\t\t\t\"root\"\t\t\t\t\t\t\t\t\t\tChanges the current directory to the ROOT dirtectory\n");
    printf("5.\t\t\t\"pwd\"\t\t\t\t\t\t\t\t\t\tPrints the current working directory\n");
    printf("6.\t\t\t\"find filename\"\t\t\t\t\t\t\t\tSearches the file with name \"filename\" in the disk\n");
    printf("7.\t\t\t\"mkdir dirname\"\t\t\t\t\t\t\t\tCreates a new folder with name \"dirname\" in the current working directory\n");
    printf("8.\t\t\t\"copy source destination\"\t\t\t\t\tCopies a file named \"source\" from PC to the disk with name \"destination\"\n");
    printf("9.\t\t\t\"read source destination\"\t\t\t\t\tReads a file named \"source\" from the disk to the PC with name '\destination\"\n");    
    printf("10.\t\t\t\"del file/directory name \"\t\t\t\t\tDeletes a file/folder with the given name from the current working directory\n");
    printf("11.\t\t\t\"exit\"\t\t\t\t\t\t\t\t\t\tTo exit the command prompt\n");
    
    printf("================================================================================================================================================\n\n");
}


//Parsing of the command given to individual arguments
void CommandInterface()
{
    int i=0;
    int j=0;
    int command;
     
        
    printf("\n\nPlease type appropriate command (type \"help\" for the list of commands) --> ");
    
	//Initializing the command and arguments to NULL   
    for(i=0;i<257;i++)
    {
        string[i]=0;
        comm[i]=0;
        string1[i]=0;
    }
    
    for(i=0;i<800;i++)
    {
        input[i]=0;
    }
    
    // Wait for input from the user
    gets(input);
     
    i=0;
    
    //Ignore inital spaces
    while(input[i]==' ')
    	i++;
    	
    //Parse the command
    while(input[i]!=NULL&&input[i]!=' ')
    {
        comm[j]=input[i];
        i++;
        j++;
    }
    comm[i]='\0';
    i++;
    
    //Ignore the spaces after command
    while(input[i]==' ')
    	i++;
    	
    //Parse the first argument
    j=0;
    while(input[i]!=NULL&&input[i]!='.')
    {
        string[j]=input[i];
        j++;
        i++;
    }
    
    
    if(input[i]=='.')
    {
        while(input[i]!=' '&&input[i]!=NULL)
        {
        	string[j]=input[i];
        	j++;
        	i++;
        }
    }
    
   //Ignore the spaces after first argument
     while(input[i]==' ')
    	i++;
    	
    j=0;
    
    //Parse the second argument
    while(input[i]!=NULL&&input[i]!='.')
    {
        string1[j]=input[i];
        j++;
        i++;
    }
    
    if(input[i]=='.')
    {
        while(input[i]!=' '&&input[i]!=NULL)
        {
        	string1[j]=input[i];
        	j++;
        	i++;
        }
    }
       
    //Set the command to appropriate integer value based on the command entered
    if(!strcmp(comm,"dir"))
    	command=1;	
    
    else if((!strcmp(comm,"cd"))&&(strcmp(string,"..")))
        	command=2;
    	
    else if((!strcmp(comm,"cd.."))||(!strcmp(comm,"cd"))&&(!strcmp(string,"..")))
    	command=3;
    
    else if(!strcmp(comm,"root"))
    	command=4;
    
    else if(!strcmp(comm,"pwd"))
    	command=5;
    	
    else if(!strcmp(comm,"find"))
    	command=6;
    
    else if(!strcmp(comm,"mkdir"))
    	command=7;
    	
   	else if(!strcmp(comm,"copy"))
    	command=8;
    	
   	else if(!strcmp(comm,"read"))
    	command=9;
    	
    else if(!strcmp(comm,"del"))
    	command=10;
   
    else if(!strcmp(comm,"exit"))
    	command=11;
    	
    else if(!strcmp(comm,"help"))
    	command=12;
    	
    else 
    	command=13; 
      
    // Take appropriate action based on the command
    CommandInterpreter(command);
}

 
    
    	
// Function which in turn calls appropriate FAT functions based on the command
void CommandInterpreter(command)
{
    switch(command)
    {
        case 1:	// List directories and files (at current working directory)
        		ListDirectory();
    			break;
        
        case 2:	// Change directory (to a folder specified by user)
        		ChangeDirectory(string);
        		break;
        
        case 3:	// Go to a directory one level up (like  cd .. in Unix)
        		TraverseUp();
        		break;
        
        case 4:	// Go to ROOT directory
        		GotoRoot();
        		printf("\nCurrent Working Directory is: ");   
    			printpath();
        		break;
        		
        case 5:	// Get current working directory (like pwd in Unix)
        	 	printf("\nCurrent Working Directory is: ");   
    			printpath();
				break;
        
		case 6:	// Search for a file or Directory
				Search(string);
				break;

        case 7:	// Create directory
        		CreateDirectory(string);
    			break;
        
        case 8: // Copy file (from PC/DSP to disk)
        		CopyFile(string,string1);
        		break;
        
        case 9: // Read file (from disk to PC/DSP)
        		ReadFile(string,string1);
        		break;

        case 10:// Erase Directory
        		Erase(string);
        		break;
        
        case 11:// Exit Program
        		halt = true;
				break;
                               	
        case 12:// Exit Program
        		help();
				break;
        
		default:// Invalid input
        			printf("Invalid input\n\n");	
        			break;
    }
}


// List Directories and Files at the current working directory
void ListDirectory()
{
    printf("\n\nList of Directories and Files in Current Directory\n");
    printf("=======================================================================\n\n");
    lsDIR();
    printf("\n\nCurrent Working Directory is::");
    printpath();
}


// Change Directory to the folder specified
void ChangeDirectory(char* string)
{
    changeDIR(string);
    printf("\nCurrent Working Directory is: ");   
    printpath();
}

// Go to a directory that is level up the current directory
void TraverseUp(void)
{
    traverseupDIR();
    printf("\nCurrent Working Directory is: ");   
    printpath();
}

void GotoRoot()
{
    gotoROOTDIR();
}

// Search for a File or Directory
void Search(char* string)
{
    searchFILE(string);
}

// Create Directory
void CreateDirectory(char* string)
{
    int temp=createDIR(string);
    if(temp==1)
    	printf("\n\nDirectory %s created successfully\n\n",string);    
}


// Copy File
void CopyFile(char* source,char* destination)
{    
    int success=copyFILE(source,destination);
    if(success)
    	printf("\n\nFile %s from PC copied to the MMC as %s \n\n",source,destination);    
}


// Read File
void ReadFile(char* source, char* destination)
{
    int success=readFILE(source,destination);
    if(success)
    	printf("\n\nFile %s read from the MMC and copied to the local disk as %s\n\n",source,destination);    
}


// Erase
void Erase(char* string)
{
    erase_FILE_DIR(string);
     
}


